"use strict";
var _YfmEditor_view, _YfmEditor_serializer, _YfmEditor_parser, _YfmEditor_actions, _YfmEditor_contentHandler;
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmEditor = void 0;
const tslib_1 = require("tslib");
const prosemirror_view_1 = require("prosemirror-view");
const prosemirror_state_1 = require("prosemirror-state");
const actions_1 = require("./utils/actions");
const ExtensionsManager_1 = require("./ExtensionsManager");
const ContentHandler_1 = require("./ContentHandler");
const metrics_1 = require("./utils/metrics");
class YfmEditor {
    constructor({ domElem, initialContent = '', extensions = () => { }, attrs: attrsOpts, allowHTML, linkify, linkifyTlds, onChange, onDocChange, }) {
        _YfmEditor_view.set(this, void 0);
        _YfmEditor_serializer.set(this, void 0);
        _YfmEditor_parser.set(this, void 0);
        _YfmEditor_actions.set(this, void 0);
        _YfmEditor_contentHandler.set(this, void 0);
        const { schema, parser, serializer, nodeViews, markViews, plugins, rawActions, actions } = ExtensionsManager_1.ExtensionsManager.process(extensions, {
            // "breaks" option only affects the renderer, but not the parser
            mdOpts: { html: allowHTML, linkify, breaks: true },
            attrsOpts: Object.assign(Object.assign({}, attrsOpts), { allowedAttributes: ['id'] }),
            linkifyTlds,
        });
        const state = prosemirror_state_1.EditorState.create({
            schema,
            doc: parser.parse(initialContent),
            plugins,
        });
        const thisOnChange = () => this.tryOnChange(onChange);
        const thisOnDocChange = () => this.tryOnChange(onDocChange);
        tslib_1.__classPrivateFieldSet(this, _YfmEditor_view, new prosemirror_view_1.EditorView(domElem !== null && domElem !== void 0 ? domElem : null, {
            state,
            nodeViews,
            markViews,
            dispatchTransaction(tr) {
                const newState = this.state.apply(tr);
                // @ts-expect-error
                this.updateState(newState);
                thisOnChange();
                if (tr.docChanged) {
                    thisOnDocChange();
                }
                (0, metrics_1.logTransactionMetrics)(tr);
            },
        }), "f");
        tslib_1.__classPrivateFieldSet(this, _YfmEditor_actions, actions.setActions((0, actions_1.bindActions)(rawActions)(tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f"))), "f");
        tslib_1.__classPrivateFieldSet(this, _YfmEditor_serializer, serializer, "f");
        tslib_1.__classPrivateFieldSet(this, _YfmEditor_parser, parser, "f");
        tslib_1.__classPrivateFieldSet(this, _YfmEditor_contentHandler, new ContentHandler_1.WysiwygContentHandler(tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f"), parser), "f");
    }
    get dom() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").dom;
    }
    get serializer() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_serializer, "f");
    }
    get parser() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_parser, "f");
    }
    get actions() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_actions, "f").actions;
    }
    /** @internal used for prosemirror-dev-tools in demo */
    get view() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f");
    }
    action(actionName) {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_actions, "f").action(actionName);
    }
    focus() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").focus();
    }
    hasFocus() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").hasFocus();
    }
    getValue() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_serializer, "f").serialize(tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").state.doc);
    }
    isEmpty() {
        var _a;
        const { doc } = tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").state;
        return (doc.childCount === 1 &&
            ((_a = doc.firstChild) === null || _a === void 0 ? void 0 : _a.type.name) === 'paragraph' &&
            doc.firstChild.childCount === 0);
    }
    clear() {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_contentHandler, "f").clear();
    }
    replace(newMarkup) {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_contentHandler, "f").replace(newMarkup);
    }
    prepend(markup) {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_contentHandler, "f").prepend(markup);
    }
    append(markup) {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_contentHandler, "f").append(markup);
    }
    moveCursor(position) {
        return tslib_1.__classPrivateFieldGet(this, _YfmEditor_contentHandler, "f").moveCursor(position);
    }
    destroy() {
        tslib_1.__classPrivateFieldGet(this, _YfmEditor_view, "f").destroy();
    }
    tryOnChange(cb) {
        if (cb) {
            try {
                cb(this);
            }
            catch (err) {
                console.error(err);
            }
        }
    }
}
exports.YfmEditor = YfmEditor;
_YfmEditor_view = new WeakMap(), _YfmEditor_serializer = new WeakMap(), _YfmEditor_parser = new WeakMap(), _YfmEditor_actions = new WeakMap(), _YfmEditor_contentHandler = new WeakMap();
