import MarkdownIt from 'markdown-it';
import type { Plugin } from 'prosemirror-state';
import { ActionsManager } from './ActionsManager';
import type { ActionSpec } from './types/actions';
import type { Extension, ExtensionDeps } from './types/extension';
import type { MarkViewConstructor, NodeViewConstructor } from './types/node-views';
declare type ExtensionsManagerParams = {
    extensions: Extension;
    options?: ExtensionsManagerOptions;
};
declare type ExtensionsManagerOptions = {
    mdOpts?: MarkdownIt.Options;
    attrsOpts?: {
        leftDelimiter?: string;
        rightDelimiter?: string;
        allowedAttributes?: string[];
    };
    linkifyTlds?: string | string[];
};
export declare class ExtensionsManager {
    #private;
    static process(extensions: Extension, options: ExtensionsManagerOptions): {
        actions: ActionsManager;
        rawActions: Record<string, ActionSpec>;
        plugins: Plugin<any>[];
        nodeViews: Record<string, NodeViewConstructor>;
        markViews: Record<string, MarkViewConstructor>;
        schema: import("prosemirror-model").Schema<any, any>;
        parser: import(".").Parser;
        parserWithoutAttrs: import(".").Parser;
        serializer: import(".").Serializer;
    };
    constructor({ extensions, options }: ExtensionsManagerParams);
    build(): {
        actions: ActionsManager;
        rawActions: Record<string, ActionSpec>;
        plugins: Plugin<any>[];
        nodeViews: Record<string, NodeViewConstructor>;
        markViews: Record<string, MarkViewConstructor>;
        schema: import("prosemirror-model").Schema<any, any>;
        parser: import(".").Parser;
        parserWithoutAttrs: import(".").Parser;
        serializer: import(".").Serializer;
    };
    buildDeps(): ExtensionDeps;
    private processExtensions;
    private processNode;
    private processMark;
    private createDeps;
    private createDerived;
}
export {};
