"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBlockSpecs = exports.codeBlockType = exports.codeBlockLangAttr = exports.codeBlockNodeName = void 0;
const schema_1 = require("../../../../utils/schema");
exports.codeBlockNodeName = 'code_block';
exports.codeBlockLangAttr = 'data-language';
exports.codeBlockType = (0, schema_1.nodeTypeFactory)(exports.codeBlockNodeName);
const CodeBlockSpecs = (builder, opts) => {
    builder.addNode(exports.codeBlockNodeName, () => ({
        view: opts.nodeview,
        spec: {
            attrs: { [exports.codeBlockLangAttr]: { default: 'text' } },
            content: 'text*',
            group: 'block',
            code: true,
            marks: '',
            selectable: true,
            allowSelection: true,
            parseDOM: [
                {
                    tag: 'pre',
                    preserveWhitespace: 'full',
                    getAttrs: (node) => ({
                        [exports.codeBlockLangAttr]: node.getAttribute(exports.codeBlockLangAttr) || '',
                    }),
                },
            ],
            toDOM({ attrs }) {
                return ['pre', attrs[exports.codeBlockLangAttr] ? attrs : {}, ['code', 0]];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: exports.codeBlockNodeName,
                type: 'block',
                noCloseToken: true,
                prepareContent: removeNewLineAtEnd, // content of code blocks contains extra \n at the end
            },
        },
        toYfm: (state, node) => {
            state.write('```' + (node.attrs[exports.codeBlockLangAttr] || '') + '\n');
            state.text(node.textContent, false);
            // Add a newline to the current content before adding closing marker
            state.write('\n');
            state.write('```');
            state.closeBlock(node);
        },
    }));
    builder.addNode('fence', () => ({
        //  we adding this node only for define specific 'fence' parser token,
        //  which parse fence md token to code_block node
        spec: {},
        fromYfm: {
            tokenSpec: {
                name: exports.codeBlockNodeName,
                type: 'block',
                noCloseToken: true,
                getAttrs: (tok) => ({ [exports.codeBlockLangAttr]: tok.info || '' }),
                prepareContent: removeNewLineAtEnd, // content of fence blocks contains extra \n at the end
            },
        },
        toYfm: () => {
            throw new Error('Unexpected toYfm() call on fence node');
        },
    }));
};
exports.CodeBlockSpecs = CodeBlockSpecs;
function removeNewLineAtEnd(content) {
    return content.endsWith('\n') ? content.slice(0, content.length - 1) : content;
}
