"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlePaste = void 0;
const utils_1 = require("../../behavior/Clipboard/utils");
const const_1 = require("./const");
const handlePaste = (view, e) => {
    if (!e.clipboardData || view.state.selection.$from.parent.type.spec.code)
        return false;
    const code = getCodeData(e.clipboardData);
    if (!code)
        return false;
    let tr = view.state.tr;
    const { schema } = tr.doc.type;
    const codeBlockNode = (0, const_1.cbType)(schema).create({ [const_1.codeBlockLangAttr]: code.mode }, schema.text(code.value));
    tr = tr.replaceSelectionWith(codeBlockNode);
    view.dispatch(tr.scrollIntoView());
    return true;
};
exports.handlePaste = handlePaste;
function getCodeData(data) {
    var _a;
    if (data.getData(utils_1.DataTransferType.Text)) {
        let editor = 'unknown';
        let mode;
        if (isVSCode(data)) {
            editor = 'vscode';
            mode = (_a = tryCatch(() => JSON.parse(data.getData(utils_1.DataTransferType.VSCodeData)))) === null || _a === void 0 ? void 0 : _a.mode;
        }
        else
            return null;
        return { editor, mode, value: data.getData(utils_1.DataTransferType.Text) };
    }
    return null;
}
function isVSCode(data) {
    return data.types.includes(utils_1.DataTransferType.VSCodeData);
}
function tryCatch(fn) {
    try {
        return fn();
    }
    catch (e) {
        console.error(e);
    }
    return undefined;
}
