"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lists = exports.olType = exports.liType = exports.blType = exports.ListNode = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_schema_list_1 = require("prosemirror-schema-list");
const keymap_1 = require("../../../utils/keymap");
const actions_1 = require("./actions");
const const_1 = require("./const");
const inputrules_1 = require("./inputrules");
const ListsSpecs_1 = require("./ListsSpecs");
const commands_1 = require("./commands");
var ListsSpecs_2 = require("./ListsSpecs");
Object.defineProperty(exports, "ListNode", { enumerable: true, get: function () { return ListsSpecs_2.ListNode; } });
Object.defineProperty(exports, "blType", { enumerable: true, get: function () { return ListsSpecs_2.blType; } });
Object.defineProperty(exports, "liType", { enumerable: true, get: function () { return ListsSpecs_2.liType; } });
Object.defineProperty(exports, "olType", { enumerable: true, get: function () { return ListsSpecs_2.olType; } });
const Lists = (builder, opts) => {
    builder.use(ListsSpecs_1.ListsSpecs);
    builder.addKeymap(({ schema }) => {
        const { ulKey, olKey } = opts !== null && opts !== void 0 ? opts : {};
        const bindings = {};
        if (ulKey)
            bindings[ulKey] = (0, keymap_1.withLogAction)('bulletList', (0, commands_1.toList)((0, ListsSpecs_1.blType)(schema)));
        if (olKey)
            bindings[olKey] = (0, keymap_1.withLogAction)('orderedList', (0, commands_1.toList)((0, ListsSpecs_1.olType)(schema)));
        return Object.assign({ Enter: (0, prosemirror_schema_list_1.splitListItem)((0, ListsSpecs_1.liType)(schema)), Tab: (0, prosemirror_schema_list_1.sinkListItem)((0, ListsSpecs_1.liType)(schema)), 'Shift-Tab': (0, prosemirror_schema_list_1.liftListItem)((0, ListsSpecs_1.liType)(schema)), 'Mod-[': (0, prosemirror_schema_list_1.liftListItem)((0, ListsSpecs_1.liType)(schema)), 'Mod-]': (0, prosemirror_schema_list_1.sinkListItem)((0, ListsSpecs_1.liType)(schema)) }, bindings);
    });
    builder.addKeymap(() => ({
        Backspace: (0, prosemirror_commands_1.chainCommands)(commands_1.liftIfCursorIsAtBeginningOfItem, commands_1.joinPrevList),
    }), builder.Priority.Low);
    builder.use(inputrules_1.ListsInputRulesExtension, { bulletListInputRule: opts === null || opts === void 0 ? void 0 : opts.ulInputRules });
    builder
        .addAction(const_1.ListAction.ToBulletList, actions_1.actions.toBulletList)
        .addAction(const_1.ListAction.ToOrderedList, actions_1.actions.toOrderedList)
        .addAction(const_1.ListAction.SinkListItem, actions_1.actions.sinkListItem)
        .addAction(const_1.ListAction.LiftListItem, actions_1.actions.liftListItem);
};
exports.Lists = Lists;
