"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCheckbox = exports.addCheckboxCmd = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const BaseSchema_1 = require("../../base/BaseSchema");
const utils_1 = require("./utils");
const createCheckbox = (schema, content) => (0, utils_1.checkboxType)(schema).create({}, [
    (0, utils_1.checkboxInputType)(schema).create(),
    (0, utils_1.checkboxLabelType)(schema).create({}, content),
]);
const addCheckboxCmd = (state, dispatch) => {
    const paragraph = (0, prosemirror_utils_1.findParentNodeOfType)((0, BaseSchema_1.pType)(state.schema))(state.selection);
    const checkboxParent = (0, prosemirror_utils_1.findParentNodeOfType)((0, utils_1.checkboxType)(state.schema))(state.selection);
    const parent = paragraph || checkboxParent;
    if (!parent)
        return false;
    const checkboxChild = (0, prosemirror_utils_1.findChildrenByType)(parent.node, (0, utils_1.checkboxLabelType)(state.schema));
    if (checkboxChild.length) {
        if (dispatch) {
            const { tr } = state;
            tr.insert(parent.pos + parent.node.nodeSize, createCheckbox(state.schema, undefined));
            tr.setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(tr.selection.$from.after() + 4)));
            dispatch(tr);
        }
        return true;
    }
    const { tr } = state;
    if (dispatch) {
        tr.replaceWith(parent.pos, parent.pos + parent.node.nodeSize, createCheckbox(state.schema, parent.node.content));
        tr.setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(tr.selection.$from.after() - 1)));
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr);
    }
    return true;
};
exports.addCheckboxCmd = addCheckboxCmd;
const addCheckbox = () => {
    return {
        isEnable: exports.addCheckboxCmd,
        run: exports.addCheckboxCmd,
    };
};
exports.addCheckbox = addCheckbox;
