"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImgSizeSpecs = exports.ImgSizeAttr = void 0;
const tslib_1 = require("tslib");
const is_number_1 = tslib_1.__importDefault(require("is-number"));
const imsize_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/plugins/imsize"));
const const_1 = require("@doc-tools/transform/lib/plugins/imsize/const");
Object.defineProperty(exports, "ImgSizeAttr", { enumerable: true, get: function () { return const_1.ImsizeAttr; } });
const log_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/log"));
const const_2 = require("../../../markdown/Image/const");
const ImgSizeSpecs = (builder, opts) => {
    builder.configureMd((md) => md.use(imsize_1.default, { log: log_1.default }));
    builder.addNode(const_2.imageNodeName, () => ({
        spec: {
            inline: true,
            attrs: {
                [const_1.ImsizeAttr.Src]: {},
                [const_1.ImsizeAttr.Alt]: { default: null },
                [const_1.ImsizeAttr.Title]: { default: null },
                [const_1.ImsizeAttr.Height]: { default: null },
                [const_1.ImsizeAttr.Width]: { default: null },
            },
            placeholder: opts.placeholder,
            group: 'inline',
            draggable: true,
            parseDOM: [
                {
                    tag: 'img[src]',
                    getAttrs(dom) {
                        const height = dom.getAttribute(const_1.ImsizeAttr.Height);
                        const width = dom.getAttribute(const_1.ImsizeAttr.Width);
                        return {
                            [const_1.ImsizeAttr.Src]: dom.getAttribute(const_1.ImsizeAttr.Src),
                            [const_1.ImsizeAttr.Alt]: dom.getAttribute(const_1.ImsizeAttr.Alt),
                            [const_1.ImsizeAttr.Title]: dom.getAttribute(const_1.ImsizeAttr.Title),
                            [const_1.ImsizeAttr.Height]: (0, is_number_1.default)(height) ? height : null,
                            [const_1.ImsizeAttr.Width]: (0, is_number_1.default)(width) ? height : null,
                        };
                    },
                },
            ],
            toDOM(node) {
                return ['img', node.attrs];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_2.imageNodeName,
                type: 'node',
                getAttrs: (tok) => {
                    var _a, _b;
                    return ({
                        [const_1.ImsizeAttr.Src]: tok.attrGet(const_1.ImsizeAttr.Src),
                        [const_1.ImsizeAttr.Title]: tok.attrGet(const_1.ImsizeAttr.Title),
                        [const_1.ImsizeAttr.Height]: tok.attrGet(const_1.ImsizeAttr.Height),
                        [const_1.ImsizeAttr.Width]: tok.attrGet(const_1.ImsizeAttr.Width),
                        [const_1.ImsizeAttr.Alt]: ((_b = (_a = tok.children) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.content) || null,
                    });
                },
            },
        },
        toYfm: (state, node) => {
            const attrs = node.attrs;
            state.write('![' +
                state.esc(attrs.alt || '') +
                '](' +
                state.esc(attrs.src) +
                (attrs.title ? ' ' + state.quote(attrs.title) : '') +
                getSize(attrs) +
                ')');
        },
    }));
};
exports.ImgSizeSpecs = ImgSizeSpecs;
function getSize({ width, height }) {
    if (width || height) {
        return ` =${width || ''}x${height || ''}`;
    }
    return '';
}
