"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmHeadingSpecs = exports.YfmHeadingAttr = void 0;
const const_1 = require("./const");
const utils_1 = require("./utils");
const DEFAULT_PLACEHOLDER = (node) => 'Heading ' + node.attrs[const_1.YfmHeadingAttr.Level];
var const_2 = require("./const");
Object.defineProperty(exports, "YfmHeadingAttr", { enumerable: true, get: function () { return const_2.YfmHeadingAttr; } });
/** YfmHeading extension needs markdown-it-attrs plugin */
const YfmHeadingSpecs = (builder, opts) => {
    const { headingPlaceholder } = opts !== null && opts !== void 0 ? opts : {};
    builder.addNode(const_1.headingNodeName, () => ({
        spec: {
            attrs: {
                [const_1.YfmHeadingAttr.Id]: { default: '' },
                [const_1.YfmHeadingAttr.Level]: { default: 1 },
            },
            content: '(text | inline)*',
            group: 'block',
            defining: true,
            selectable: true,
            parseDOM: [
                { tag: 'h1', getAttrs: (0, utils_1.getNodeAttrs)(1) },
                { tag: 'h2', getAttrs: (0, utils_1.getNodeAttrs)(2) },
                { tag: 'h3', getAttrs: (0, utils_1.getNodeAttrs)(3) },
                { tag: 'h4', getAttrs: (0, utils_1.getNodeAttrs)(4) },
                { tag: 'h5', getAttrs: (0, utils_1.getNodeAttrs)(5) },
                { tag: 'h6', getAttrs: (0, utils_1.getNodeAttrs)(6) },
            ],
            toDOM(node) {
                const id = node.attrs[const_1.YfmHeadingAttr.Id];
                return [
                    'h' + node.attrs[const_1.YfmHeadingAttr.Level],
                    id ? { id } : {},
                    0,
                    // [
                    //     'a',
                    //     {
                    //         href: `#${node.attrs[YfmHeadingAttr.Id]}`,
                    //         class: 'yfm-anchor',
                    //         'aria-hidden': 'true',
                    //         contenteditable: 'false',
                    //     },
                    // ],
                    // ['span', 0],
                ];
            },
            placeholder: {
                content: headingPlaceholder !== null && headingPlaceholder !== void 0 ? headingPlaceholder : DEFAULT_PLACEHOLDER,
                alwaysVisible: true,
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_1.headingNodeName,
                type: 'block',
                getAttrs: (token) => {
                    if (token.type.endsWith('_close'))
                        return {};
                    const attrs = Object.fromEntries(token.attrs || []);
                    // if (!attrs[YfmHeadingAttr.Id]) {
                    //     // calculate id if it was not specified
                    //     // tokens[index + 1] is child inline token
                    //     attrs[YfmHeadingAttr.Id] = slugify(tokens[index + 1].content);
                    // }
                    // attrs have id only if it explicitly specified manually
                    return Object.assign({ [const_1.YfmHeadingAttr.Level]: Number(token.tag.slice(1)) }, attrs);
                },
            },
        },
        toYfm: (state, node) => {
            state.write(state.repeat('#', node.attrs[const_1.YfmHeadingAttr.Level]) + ' ');
            state.renderInline(node);
            const anchor = node.attrs[const_1.YfmHeadingAttr.Id];
            if (anchor /*&& anchor !== node.firstChild?.textContent*/) {
                state.write(` {#${anchor}}`);
            }
            state.closeBlock(node);
        },
    }));
};
exports.YfmHeadingSpecs = YfmHeadingSpecs;
