"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.joinBackwardToOpenTab = exports.removeTabWhenCursorAtTheStartOfTab = exports.removeTab = exports.createTab = exports.tabEnter = exports.tabPanelArrowDown = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const const_1 = require("./const");
const helpers_1 = require("../../../table-utils/helpers");
const selection_1 = require("../../../utils/selection");
const utils_1 = require("@doc-tools/transform/lib/plugins/utils");
const __1 = require("../../");
const utils_2 = require("./utils");
const const_2 = require("./YfmTabsSpecs/const");
const tabPanelArrowDown = (state, dispatch, view) => {
    var _a;
    const { selection: sel } = state;
    const tabsParentNode = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_1.tabsType)(state.schema))(state.selection);
    if ((0, utils_2.atEndOfPanel)(view) && (view === null || view === void 0 ? void 0 : view.endOfTextblock('down'))) {
        const [direction, $cursor] = ['after', sel.$to];
        let $pos;
        if ((0, selection_1.isTextSelection)(sel)) {
            $pos =
                (_a = (0, __1.findFakeParaPosForTextSelection)(sel, direction)) !== null && _a !== void 0 ? _a : (0, __1.findFakeParaPosClosestToPos)($cursor, $cursor.depth - 2, direction);
        }
        if ((0, __1.isGapCursorSelection)(sel) && tabsParentNode) {
            $pos = state.doc.resolve(tabsParentNode.pos + tabsParentNode.node.nodeSize);
        }
        if ($pos) {
            dispatch === null || dispatch === void 0 ? void 0 : dispatch((0, __1.createFakeParagraph)(state.tr, $pos, direction).scrollIntoView());
            return true;
        }
    }
    return false;
};
exports.tabPanelArrowDown = tabPanelArrowDown;
const tabEnter = (state) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    return ($cursor === null || $cursor === void 0 ? void 0 : $cursor.node($cursor.depth).type) === (0, const_1.tabType)(state.schema);
};
exports.tabEnter = tabEnter;
const makeTabsInactive = (tabNodes, tabPanels, tr) => {
    // Find all active tabs and make them inactive
    const activeTabs = tabNodes.filter((v) => v.node.attrs[const_2.TabAttrs.dataDiplodocIsActive] === 'true');
    if (activeTabs.length) {
        activeTabs.forEach((tab) => {
            tr.setNodeMarkup(tab.pos, null, Object.assign(Object.assign({}, tab.node.attrs), { class: const_1.tabInactiveClassname, [const_2.TabAttrs.dataDiplodocIsActive]: 'false' }));
        });
    }
    // Find all active panels and make them inactive
    const activePanels = tabPanels.filter((v) => v.node.attrs[const_2.TabPanelAttrs.class] === const_1.tabPanelActiveClassname);
    if (activePanels.length) {
        activePanels.forEach((tabPanel) => {
            tr.setNodeMarkup(tr.mapping.map(tabPanel.pos), null, Object.assign(Object.assign({}, tabPanel.node.attrs), { class: const_1.tabPanelInactiveClassname }));
        });
    }
};
const createTab = (afterTab, tabsParentNode) => (state, dispatch, view) => {
    const tabNodes = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (node) => node.type.name === (0, const_1.tabType)(state.schema).name);
    const tabPanels = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_1.tabPanelType)(state.schema).name;
    });
    const afterPanelNode = tabPanels.filter((tabPanelNode) => tabPanelNode.node.attrs[const_2.TabPanelAttrs.ariaLabelledby] ===
        afterTab.node.attrs[const_2.TabAttrs.dataDiplodocid])[0];
    const tabId = (0, utils_1.generateID)();
    const panelId = (0, utils_1.generateID)();
    const newPanel = (0, const_1.tabPanelType)(state.schema).create({
        [const_2.TabPanelAttrs.ariaLabelledby]: tabId,
        [const_2.TabPanelAttrs.id]: panelId,
        [const_2.TabPanelAttrs.class]: const_1.tabPanelActiveClassname,
    }, (0, __1.pType)(state.schema).createAndFill());
    const newTab = (0, const_1.tabType)(state.schema).create({
        [const_2.TabAttrs.id]: tabId,
        [const_2.TabAttrs.dataDiplodocid]: tabId,
        [const_2.TabAttrs.dataDiplodocKey]: tabId,
        [const_2.TabAttrs.dataDiplodocIsActive]: 'true',
        [const_2.TabAttrs.class]: const_1.tabActiveClassname,
        [const_2.TabAttrs.role]: 'tab',
        [const_2.TabAttrs.ariaControls]: panelId,
    });
    const { tr } = state;
    // Change relative pos to absolute
    tabNodes.forEach((v) => {
        v.pos = v.pos + tabsParentNode.pos + 1;
    });
    tabPanels.forEach((v) => {
        v.pos = v.pos + tabsParentNode.pos + 1;
    });
    makeTabsInactive(tabNodes, tabPanels, tr);
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr
        .insert(afterPanelNode.pos + afterPanelNode.node.nodeSize, newPanel)
        .insert(afterTab.pos + afterTab.node.nodeSize, newTab)
        .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, afterTab.pos + afterTab.node.nodeSize + 1)));
    view === null || view === void 0 ? void 0 : view.focus();
    return true;
};
exports.createTab = createTab;
const removeTab = (tabToRemove, tabsParentNode) => (state, dispatch, view) => {
    const tabList = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_1.tabsListType)(state.schema).name;
    })[0];
    const tabToRemoveIdx = (0, helpers_1.findChildIndex)(tabList.node, tabToRemove.node);
    const tabNodes = (0, prosemirror_utils_1.findChildren)(tabList.node, (node) => node.type.name === (0, const_1.tabType)(state.schema).name);
    const tabPanels = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
        return tabNode.type.name === (0, const_1.tabPanelType)(state.schema).name;
    });
    const panelToRemove = tabPanels.filter((tabPanelNode) => tabPanelNode.node.attrs[const_2.TabPanelAttrs.ariaLabelledby] ===
        tabToRemove.node.attrs[const_2.TabAttrs.dataDiplodocid])[0];
    if (panelToRemove && dispatch) {
        const { tr } = state;
        if (tabNodes.length <= 1) {
            tr.delete(tabsParentNode.pos, tabsParentNode.pos + tabsParentNode.node.nodeSize);
        }
        else {
            const newTabIdx = tabToRemoveIdx - 1 < 0 ? 1 : tabToRemoveIdx - 1;
            // Change relative pos to absolute
            tabNodes.forEach((v) => {
                v.pos = v.pos + tabsParentNode.pos + 2;
            });
            tabPanels.forEach((v) => {
                v.pos = v.pos + tabsParentNode.pos + 1;
            });
            const newTabNode = tabNodes[newTabIdx];
            const newTabPanelNode = tabPanels[newTabIdx];
            makeTabsInactive(tabNodes, tabPanels, tr);
            tr
                // Delete panel
                .delete(panelToRemove.pos, panelToRemove.pos + panelToRemove.node.nodeSize)
                // Delete tab
                .delete(tabToRemove.pos, tabToRemove.pos + tabToRemove.node.nodeSize)
                // Set new active tab
                .setNodeMarkup(tr.mapping.map(newTabNode.pos), null, Object.assign(Object.assign({}, newTabNode.node.attrs), { class: const_1.tabActiveClassname, [const_2.TabAttrs.dataDiplodocIsActive]: 'true' }))
                // Set new active panel
                .setNodeMarkup(tr.mapping.map(newTabPanelNode.pos), null, Object.assign(Object.assign({}, newTabPanelNode.node.attrs), { class: const_1.tabPanelActiveClassname }))
                .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, tr.mapping.map(newTabNode.pos + newTabNode.node.nodeSize - 1)));
        }
        dispatch(tr);
        view === null || view === void 0 ? void 0 : view.focus();
        return true;
    }
    return false;
};
exports.removeTab = removeTab;
const removeTabWhenCursorAtTheStartOfTab = (state, dispatch) => {
    const tabToRemove = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_1.tabType)(state.schema))(state.selection);
    const tabsParentNode = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_1.tabsType)(state.schema))(state.selection);
    if (tabsParentNode &&
        tabToRemove &&
        state.selection.from === tabToRemove.pos + 1 &&
        state.selection.from === state.selection.to) {
        return (0, exports.removeTab)(tabToRemove, tabsParentNode)(state, dispatch);
    }
    return false;
};
exports.removeTabWhenCursorAtTheStartOfTab = removeTabWhenCursorAtTheStartOfTab;
const joinBackwardToOpenTab = (state, dispatch) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (!$cursor || $cursor.parentOffset !== 0)
        return false;
    // --> cursor at start of textblock
    const textBlockIndex = $cursor.index($cursor.depth - 1);
    if (textBlockIndex <= 0)
        return false;
    const nodeBefore = $cursor.node($cursor.depth - 1).child(textBlockIndex - 1);
    if (nodeBefore.type !== (0, const_1.tabsType)(state.schema)) {
        return false;
    }
    const tabsParent = (0, prosemirror_utils_1.findParentNodeOfTypeClosestToPos)(state.doc.resolve($cursor.pos - 2), (0, const_1.tabsType)(state.schema));
    if (!tabsParent) {
        return false;
    }
    const activePanel = (0, prosemirror_utils_1.findChildren)(tabsParent.node, (n) => n.attrs.class === const_1.tabPanelActiveClassname)[0];
    if (dispatch) {
        const posEndOfLastLayoutCell = activePanel.pos + tabsParent.pos + activePanel.node.nodeSize;
        const tr = state.tr;
        tr.delete($cursor.before(), $cursor.after());
        tr.insert(posEndOfLastLayoutCell, $cursor.parent);
        tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, posEndOfLastLayoutCell + 1));
        dispatch(tr.scrollIntoView());
    }
    return true;
};
exports.joinBackwardToOpenTab = joinBackwardToOpenTab;
