"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarListButton = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const lodash_1 = require("lodash");
const uikit_1 = require("@gravity-ui/uikit");
const components_1 = require("@gravity-ui/components");
const icons_1 = require("@gravity-ui/icons");
const classname_1 = require("../classname");
const hooks_1 = require("../react-utils/hooks");
const const_1 = require("./const");
const b = (0, classname_1.cn)('toolbar-list-button');
function ToolbarListButton({ className, editor, focus, onClick, icon, title, withArrow, data, }) {
    var _a;
    const buttonRef = react_1.default.useRef(null);
    const [open, , hide, toggleOpen] = (0, hooks_1.useBooleanState)(false);
    const someActive = data.some((item) => item.isActive(editor));
    const everyDisabled = data.every((item) => !item.isEnable(editor));
    const popupOpen = everyDisabled ? false : open;
    const shouldForceHide = open && !popupOpen;
    react_1.default.useEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    if (data.length === 0)
        return null;
    const buttonContent = [react_1.default.createElement(uikit_1.Icon, { key: 1, data: icon.data, size: (_a = icon.size) !== null && _a !== void 0 ? _a : 16 })];
    if (withArrow) {
        buttonContent.push(react_1.default.createElement(react_1.default.Fragment, { key: 2 }, ''));
        buttonContent.push(react_1.default.createElement(uikit_1.Icon, { key: 3, data: icons_1.ChevronDown, size: 16 }));
    }
    const titleText = (0, lodash_1.isFunction)(title) ? title() : title;
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(uikit_1.Tooltip, { content: titleText, disabled: popupOpen, openDelay: const_1.ToolbarTooltipDelay.Open, closeDelay: const_1.ToolbarTooltipDelay.Close },
            react_1.default.createElement(uikit_1.Button, { size: "m", ref: buttonRef, view: someActive || popupOpen ? 'normal' : 'flat', selected: someActive, disabled: everyDisabled, className: b({ arrow: withArrow }, [className]), onClick: toggleOpen }, buttonContent)),
        react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide },
            react_1.default.createElement(uikit_1.Menu, { size: "l", className: b('menu') }, data.map(({ id, title, icon, hotkey, isActive, isEnable, exec, hint }) => {
                var _a;
                const titleText = (0, lodash_1.isFunction)(title) ? title() : title;
                const hintText = (0, lodash_1.isFunction)(hint) ? hint() : hint;
                return (react_1.default.createElement(uikit_1.Menu.Item, { key: id, active: isActive(editor), disabled: !isEnable(editor), onClick: () => {
                        hide();
                        focus();
                        exec(editor);
                        onClick === null || onClick === void 0 ? void 0 : onClick(id);
                    }, icon: react_1.default.createElement(uikit_1.Icon, { data: icon.data, size: (_a = icon.size) !== null && _a !== void 0 ? _a : 16 }), extraProps: { 'aria-label': titleText } },
                    react_1.default.createElement("div", { className: b('item') },
                        titleText,
                        react_1.default.createElement("div", { className: b('extra') },
                            hotkey && react_1.default.createElement(uikit_1.Hotkey, { value: hotkey }),
                            hintText && (react_1.default.createElement(components_1.HelpPopover, { className: b('hint'), content: hintText }))))));
            })))));
}
exports.ToolbarListButton = ToolbarListButton;
