import type MarkdownIt from 'markdown-it';
import type { Plugin } from 'prosemirror-state';
import { inputRules } from 'prosemirror-inputrules';
import type { Keymap } from './types/keymap';
import type { ActionSpec } from './types/actions';
import type { Extension, ExtensionWithOptions, ExtensionDeps, ExtensionSpec, YEMarkSpec, YENodeSpec } from './types/extension';
declare type InputRulesConfig = Parameters<typeof inputRules>[0];
declare type ConfigureMdCallback = (md: MarkdownIt) => MarkdownIt;
declare type AddPmNodeCallback = () => YENodeSpec;
declare type AddPmMarkCallback = () => YEMarkSpec;
declare type AddPmPluginCallback = (deps: ExtensionDeps) => Plugin | Plugin[];
declare type AddPmKeymapCallback = (deps: ExtensionDeps) => Keymap;
declare type AddPmInputRulesCallback = (deps: ExtensionDeps) => InputRulesConfig;
declare type AddActionCallback = (deps: ExtensionDeps) => ActionSpec;
declare enum Priority {
    Highest = 1000000,
    VeryHigh = 100000,
    High = 10000,
    Medium = 1000,
    Low = 100,
    VeryLow = 10,
    Lowest = 0
}
declare type BuilderContext<T extends object> = {
    has(key: keyof T): boolean;
    get<K extends keyof T>(key: K): T[K] | undefined;
    set<K extends keyof T>(key: K, value: T[K]): BuilderContext<T>;
};
declare global {
    namespace YfmEditor {
        interface Context {
        }
    }
}
export declare class ExtensionBuilder {
    #private;
    static createContext(): BuilderContext<YfmEditor.Context>;
    static readonly Priority: typeof Priority;
    readonly Priority: typeof Priority;
    /** @deprecated use `ExtensionBuilder.Priority` instead */
    static readonly PluginPriority: typeof Priority;
    /** @deprecated use `builder.Priority` instead */
    readonly PluginPriority: typeof Priority;
    readonly context: BuilderContext<YfmEditor.Context>;
    constructor(context?: BuilderContext<YfmEditor.Context>);
    use(extension: Extension): this;
    use<T>(extension: ExtensionWithOptions<T>, options: T): this;
    configureMd(cb: ConfigureMdCallback): this;
    addNode(name: string, cb: AddPmNodeCallback): this;
    addMark(name: string, cb: AddPmMarkCallback, priority?: Priority): this;
    addPlugin(cb: AddPmPluginCallback, priority?: Priority): this;
    addKeymap(cb: AddPmKeymapCallback, priority?: Priority): this;
    addInputRules(cb: AddPmInputRulesCallback, priority?: Priority): this;
    addAction(name: string, cb: AddActionCallback): this;
    build(): ExtensionSpec;
}
export {};
