import { chainCommands, wrapIn } from 'prosemirror-commands';
import { wrappingInputRule } from 'prosemirror-inputrules';
import { hasParentNodeOfType } from 'prosemirror-utils';
import { withLogAction } from '../../../utils/keymap';
import { liftFromQuote, toggleQuote, joinPrevQuote } from './commands';
import { BlockquoteSpecs, blockquoteType } from './BlockquoteSpecs';
export { blockquote, blockquoteNodeName, blockquoteType } from './const';
const bqAction = 'quote';
export const Blockquote = (builder, opts) => {
    builder.use(BlockquoteSpecs);
    if (opts === null || opts === void 0 ? void 0 : opts.qouteKey) {
        const { qouteKey } = opts;
        builder.addKeymap(({ schema }) => ({
            [qouteKey]: withLogAction('blockquote', wrapIn(blockquoteType(schema))),
        }));
    }
    builder.addKeymap(() => ({
        Backspace: chainCommands(liftFromQuote, joinPrevQuote),
    }));
    builder.addInputRules(({ schema }) => ({ rules: [blockQuoteRule(blockquoteType(schema))] }));
    builder.addAction(bqAction, ({ schema }) => {
        const bq = blockquoteType(schema);
        return {
            isActive: (state) => hasParentNodeOfType(bq)(state.selection),
            isEnable: toggleQuote,
            run: toggleQuote,
        };
    });
};
/**
 * Given a blockquote node type, returns an input rule that turns `"> "`
 * at the start of a textblock into a blockquote.
 */
function blockQuoteRule(nodeType) {
    return wrappingInputRule(/^\s*>\s$/, nodeType);
}
