import { chainCommands, exitCode } from 'prosemirror-commands';
import { logger } from '../../../logger';
import { isMac } from '../../../utils/platform';
import { BreaksSpecs, hbType, sbType } from './BreaksSpecs';
export { BreaksSpecs, BreakNodeName, hbType, sbType } from './BreaksSpecs';
export const Breaks = (builder, opts) => {
    var _a;
    builder.use(BreaksSpecs);
    let preferredBreak;
    if (builder.context.has('breaks')) {
        preferredBreak = builder.context.get('breaks') ? 'soft' : 'hard';
    }
    else {
        preferredBreak = (_a = opts.preferredBreak) !== null && _a !== void 0 ? _a : 'hard';
        logger.info("[Breaks extension]: Parameter 'breaks' is not defined in context; value from options is used");
    }
    builder.addKeymap(({ schema }) => {
        const cmd = addBr((preferredBreak === 'soft' ? sbType : hbType)(schema));
        const keys = {
            'Shift-Enter': cmd,
        };
        if (isMac()) {
            keys['Ctrl-Enter'] = cmd;
        }
        return keys;
    });
};
const addBr = (br) => chainCommands(exitCode, (state, dispatch) => {
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(br.create()).scrollIntoView());
    return true;
});
