import { useState, useCallback, useRef } from 'react';
import { useEffectOnce } from 'react-use';
export function useBooleanState(initial) {
    const [value, setValue] = useState(initial);
    return [
        value,
        useCallback(() => setValue(true), []),
        useCallback(() => setValue(false), []),
        useCallback(() => setValue((val) => !val), []),
    ];
}
export const useComponentWillMount = (cb) => {
    const willMount = useRef(true);
    if (willMount.current)
        cb();
    willMount.current = false;
};
export const useRenderTime = (cb) => {
    let time = 0;
    useComponentWillMount(() => {
        time = Date.now();
    });
    useEffectOnce(() => {
        cb(Date.now() - time);
    });
};
